@@ -80,7 +80,7 @@ class PortalBridge(Component):
         Declaration of private variables and initialization of
         :py:class:`component.Component` object.
         """
-        Component.__init__(self, services, config)
+        super().__init__(services, config)
         self.host = ''
         self.curTime = time.localtime()
         self.startTime = self.curTime
@@ -161,8 +161,7 @@ class PortalBridge(Component):
             time.sleep(0.5)
 
     def finalize(self, timestamp=0.0, **keywords):
-        for sim_name in list(self.sim_map.keys()):
-            sim_data = self.sim_map[sim_name]
+        for sim_data in self.sim_map.values():
             try:
                 sim_data.monitor_file.close()
                 sim_data.json_monitor_file.close()
