@@ -16,9 +16,8 @@ import signal
 import glob
 import json
 import weakref
-import inspect
 from operator import itemgetter
-from . import messages, ipsutil, component
+from . import messages, ipsutil
 from .configobj import ConfigObj
 from .cca_es_spec import initialize_event_service
 from .ips_es_spec import eventManager
@@ -178,11 +177,12 @@ class ServicesProxy:
         self.chkpt_counter = 0
         self.sim_name = ''
         self.replay_conf = None
-        self.profile = False
         self.subflow_count = 0
         self.sub_flows = {}
         self.binary_fullpath_cache = {}
-        self.dask_preload = "dask_preload.py"
+        self.ppn = 0
+        self.cpp = 0
+        self.shared_nodes = False
 
     def __initialize__(self, component_ref):
         """
@@ -229,7 +229,7 @@ class ServicesProxy:
             elif pn_compconf.upper() == 'EXCLUSIVE':
                 self.shared_nodes = False
             else:
-                self.fwk.exception("Bad 'NODE_ALLOCATION_MODE' value %s" % pn_compconf)
+                self.fwk.error("Bad 'NODE_ALLOCATION_MODE' value %s", pn_compconf)
                 raise Exception("Bad 'NODE_ALLOCATION_MODE' value %s")
         except Exception:
             self.shared_nodes = self.sim_conf['NODE_ALLOCATION_MODE'] == 'SHARED'
@@ -242,26 +242,17 @@ class ServicesProxy:
         except Exception:
             self.ppn = 0
 
+        try:
+            self.cpp = int(conf['CPUS_PER_PROC'])
+        except Exception:
+            self.cpp = 0
+
         if self.sim_conf['SIMULATION_MODE'] == 'RESTART':
             if self.sim_conf['RESTART_TIME'] == 'LATEST':
                 chkpts = glob.glob(os.path.join(self.sim_conf['RESTART_ROOT'], 'restart', '*'))
                 base_dir = sorted(chkpts, key=lambda d: float(os.path.basename(d)))[-1]
                 self.sim_conf['RESTART_TIME'] = os.path.basename(base_dir)
 
-        # Get path to IPS modules and PYTHONPATH
-        pypath = [os.path.dirname(inspect.getabsfile(component))]
-        try:
-            pypath.extend(os.environ["PYTHONPATH"].split(":"))
-        except KeyError:
-            pass
-
-        preload_txt = "import sys;"
-        for d in pypath:
-            preload_txt += f"sys.path.insert(0,'{d}');"
-
-        self.dask_preload = os.path.join(os.getcwd(), self.dask_preload)
-        open(self.dask_preload, "w").write(preload_txt)
-
     def _init_event_service(self):
         """
         Initialize connection to the central framework event service
@@ -315,25 +306,22 @@ class ServicesProxy:
         :py:meth:`messages.ServiceResponseMessage` when available, otherwise
         ``None``.
         """
-        if msg_id in list(self.finished_calls.keys()):
-            response = self.finished_calls[msg_id]
-            del self.finished_calls[msg_id]
-            return response
-        elif msg_id not in list(self.incomplete_calls.keys()):
-            self.error('Invalid call ID : %s ', str(msg_id))
-            raise Exception('Invalid message request ID argument')
+        try:
+            return self.finished_calls.pop(msg_id)
+        except KeyError:
+            if msg_id not in self.incomplete_calls:
+                self.error('Invalid call ID : %s ', str(msg_id))
+                raise Exception('Invalid message request ID argument')
 
         keep_going = True
         while keep_going:
             # get new messages, block until something interesting comes along
             responses = self._get_incoming_responses(block)
             for r in responses:
-                if r.__class__ == messages.ServiceResponseMessage:
-                    if (r.request_msg_id not in
-                            list(self.incomplete_calls.keys())):
+                if isinstance(r, messages.ServiceResponseMessage):
+                    if r.request_msg_id not in self.incomplete_calls:
                         self.error('Mismatched service response msg_id %s',
                                    str(r.request_msg_id))
-                        #                        dumpAll()
                         raise Exception('Mismatched service response msg_id.')
                     else:
                         del self.incomplete_calls[msg_id]
@@ -351,13 +339,7 @@ class ServicesProxy:
             if not block:
                 keep_going = False
         # if this message corresponds to a finish invocation, return the response message
-        if msg_id in list(self.finished_calls.keys()):
-            response = self.finished_calls[msg_id]
-            del self.finished_calls[msg_id]
-            #            dumpAll()
-            return response
-        #        dumpAll()
-        return None
+        return self.finished_calls.pop(msg_id, None)
 
     def _invoke_service(self, component_id, method_name, *args, **keywords):
         r"""
@@ -567,6 +549,7 @@ class ServicesProxy:
         manage how the binary is launched.  Keywords may be the following:
 
             * *task_ppn* : the processes per node value for this task
+            * *task_cpp* : the cores per process, only used when ``MPIRUN=srun`` commands
             * *block* : specifies that this task will block (or raise an
               exception) if not enough resources are available to run
               immediately.  If ``True``, the task will be retried until it
@@ -611,7 +594,7 @@ class ServicesProxy:
 
         """
         if not isinstance(binary, str):
-            self.exception('Error in launch_task: task binary of wrong type, expected str but found %s', type(binary).__name__)
+            self.error('Error in launch_task: task binary of wrong type, expected str but found %s', type(binary).__name__)
             raise ValueError(f"task binary of wrong type, expected str but found {type(binary).__name__}")
 
         args = tuple(str(a) for a in args)
@@ -624,12 +607,13 @@ class ServicesProxy:
         except KeyError:
             binary_fullpath = ipsutil.which(binary)
         if not binary_fullpath:
-            self.error("Program %s is not in path or is not executable" % binary)
+            self.error("Program %s is not in path or is not executable", binary)
             raise Exception("Program %s is not in path or is not executable" % binary)
         else:
             self.binary_fullpath_cache[binary] = binary_fullpath
 
         task_ppn = keywords.get('task_ppn', self.ppn)
+        task_cpp = keywords.get('task_cpp', self.cpp)
         block = keywords.get('block', True)
         tag = keywords.get('tag', 'None')
 
@@ -641,11 +625,21 @@ class ServicesProxy:
             msg_id = self._invoke_service(self.fwk.component_id,
                                           'init_task', nproc, binary_fullpath,
                                           working_dir, task_ppn, block,
-                                          whole_nodes, whole_socks, *args)
+                                          whole_nodes, whole_socks, task_cpp, *args)
             (task_id, command, env_update) = self._get_service_response(msg_id, block=True)
         except Exception:
             raise
 
+        task_id = self._launch_task(nproc, working_dir, task_id, command, env_update, tag, keywords)
+
+        if env_update:
+            self._send_monitor_event('IPS_LAUNCH_TASK', f'task_id = {task_id} , Tag = {tag} , nproc = {nproc} , Target = {command}, env = {env_update}')
+        else:
+            self._send_monitor_event('IPS_LAUNCH_TASK', f'task_id = {task_id} , Tag = {tag} , nproc = {nproc} , Target = {command}')
+
+        return task_id
+
+    def _launch_task(self, nproc, working_dir, task_id, command, env_update, tag, keywords):
         log_filename = keywords.get('logfile')
         timeout = keywords.get("timeout", 1.e9)
 
@@ -675,7 +669,7 @@ class ServicesProxy:
         try:
             self.debug('Launching command : %s', command)
             if env_update:
-                new_env = os.environ
+                new_env = os.environ.copy()
                 new_env.update(env_update)
                 process = subprocess.Popen(cmd_lst, stdout=task_stdout,
                                            stderr=task_stderr,
@@ -688,8 +682,6 @@ class ServicesProxy:
         except Exception:
             self.exception('Error executing command : %s', command)
             raise
-        self._send_monitor_event('IPS_LAUNCH_TASK', 'task_id = %s , Tag = %s , nproc = %d , Target = %s' %
-                                 (str(task_id), tag, int(nproc), command))
 
         # FIXME: process Monitoring Command : ps --no-headers -o pid,state pid1  pid2 pid3 ...
 
@@ -714,17 +706,18 @@ class ServicesProxy:
         task_pool = self.task_pools[task_pool_name]
         queued_tasks = task_pool.queued_tasks
         submit_dict = {}
-        for (task_name, task) in queued_tasks.items():
+        for task_name, task in queued_tasks.items():
             if not isinstance(task.binary, str):
-                self.exception('Error initiating task pool %s: task %s binary of wrong type, expected str but found %s',
-                               task_pool_name, task_name, type(task.binary).__name__)
+                self.error('Error initiating task pool %s: task %s binary of wrong type, expected str but found %s',
+                           task_pool_name, task_name, type(task.binary).__name__)
                 raise ValueError(f"task {task_name} binary of wrong type, expected str but found {type(task.binary).__name__}")
             task_ppn = task.keywords.get('task_ppn', self.ppn)
             wnodes = task.keywords.get('whole_nodes', not self.shared_nodes)
             wsocks = task.keywords.get('whole_sockets', not self.shared_nodes)
+            task_cpp = task.keywords.get('task_cpp', self.cpp)
             submit_dict[task_name] = (task.nproc, task.working_dir,
                                       task.binary, task.args,
-                                      task_ppn, wnodes, wsocks)
+                                      task_ppn, wnodes, wsocks, task_cpp)
 
         try:
             msg_id = self._invoke_service(self.fwk.component_id,
@@ -735,59 +728,23 @@ class ServicesProxy:
             raise
 
         active_tasks = {}
-        for task_name in list(allocated_tasks.keys()):
+        for task_name in allocated_tasks:
             if launch_interval > 0:
                 time.sleep(launch_interval)
             task = queued_tasks[task_name]
             (task_id, command, env_update) = allocated_tasks[task_name]
-
             tag = task.keywords.get('tag', 'None')
 
-            log_filename = task.keywords.get('logfile')
+            active_tasks[task_name] = self._launch_task(task.nproc, task.working_dir, task_id, command, env_update, tag, task.keywords)
 
-            timeout = task.keywords.get("timeout", 1.e9)
-
-            task_stdout = sys.stdout
-            if log_filename:
-                try:
-                    task_stdout = open(log_filename, 'w')
-                except Exception:
-                    self.exception('Error opening log file %s : using stdout', log_filename)
-
-            task_stderr = subprocess.STDOUT
-            try:
-                err_filename = task.keywords['errfile']
-            except KeyError:
-                pass
+            if env_update:
+                self._send_monitor_event('IPS_LAUNCH_TASK_POOL',
+                                         f'task_id = {task_id} , Tag = {tag} , nproc = {task.nproc} , Target = {command} , task_name = {task_name}'
+                                         f', env = {env_update}')
             else:
-                try:
-                    task_stderr = open(err_filename, 'w')
-                except Exception:
-                    self.exception('Error opening stderr file %s : using stderr', err_filename)
+                self._send_monitor_event('IPS_LAUNCH_TASK_POOL',
+                                         f'task_id = {task_id} , Tag = {tag} , nproc = {task.nproc} , Target = {command} , task_name = {task_name}')
 
-            cmd_lst = command.split(' ')
-            try:
-                self.debug('Launching command : %s', command)
-                if env_update:
-                    new_env = os.environ
-                    new_env.update(env_update)
-                    process = subprocess.Popen(cmd_lst, stdout=task_stdout,
-                                               stderr=task_stderr,
-                                               cwd=task.working_dir,
-                                               env=new_env)
-                else:
-                    process = subprocess.Popen(cmd_lst, stdout=task_stdout,
-                                               stderr=task_stderr,
-                                               cwd=task.working_dir)
-            except Exception:
-                self.exception('Error executing task %s - command : %s', task_name, command)
-                raise
-            self._send_monitor_event('IPS_LAUNCH_TASK_POOL',
-                                     'task_id = %s , Tag = %s , nproc = %d , Target = %s , task_name = %s' %
-                                     (str(task_id), str(tag), int(task.nproc), command, task_name))
-
-            self.task_map[task_id] = (process, time.time(), timeout)
-            active_tasks[task_name] = task_id
         return active_tasks
 
     def kill_task(self, task_id):
@@ -1005,7 +962,7 @@ class ServicesProxy:
             sim_name = self.sim_name
         else:
             sim_name = target_sim_name
-        if param in list(self.sim_conf.keys()):
+        if param in self.sim_conf:
             raise Exception('Cannot dynamically alter simulation configuration parameter ' + param)
         try:
             msg_id = self._invoke_service(self.fwk.component_id,
@@ -1025,9 +982,8 @@ class ServicesProxy:
         """
         if self.time_loop is not None:
             return self.time_loop
-        sim_conf = self.sim_conf
         tlist = []
-        time_conf = sim_conf['TIME_LOOP']
+        time_conf = self.sim_conf['TIME_LOOP']
 
         def safe(nums):
             return len(set(str(nums)).difference(set("1234567890-+/*.e "))) == 0
@@ -1035,8 +991,8 @@ class ServicesProxy:
         if time_conf['MODE'] == 'REGULAR':
             for entry in ['FINISH', 'START', 'NSTEP']:
                 if not safe(time_conf[entry]):
-                    self.exception('Invalid TIME_LOOP value of %s = %s' % (entry, time_conf[entry]))
-                    raise Exception('Invalid TIME_LOOP value of %s = %s' % (entry, time_conf[entry]))
+                    self.error('Invalid TIME_LOOP value of %s = %s', entry, time_conf[entry])
+                    raise ValueError('Invalid TIME_LOOP value of %s = %s' % (entry, time_conf[entry]))
             finish = float(eval(time_conf['FINISH']))
             start = float(eval(time_conf['START']))
             nstep = int(eval(time_conf['NSTEP']))
@@ -1666,7 +1622,7 @@ class ServicesProxy:
         bin_name = merge_binary if merge_binary else "update_state"
         full_path_binary = ipsutil.which(bin_name)
         if not full_path_binary:
-            self.error("Missing executable %s in PATH" % bin_name)
+            self.error("Missing executable %s in PATH", bin_name)
             raise FileNotFoundError("Missing executable file %s in PATH" % bin_name)
         try:
             msg_id = self._invoke_service(self.fwk.component_id,
@@ -1801,7 +1757,7 @@ class ServicesProxy:
         """
         Create an empty pool of tasks with the name *task_pool_name*.  Raise exception if duplicate name.
         """
-        if task_pool_name in list(self.task_pools.keys()):
+        if task_pool_name in self.task_pools:
             raise Exception('Error: Duplicate task pool name %s' % (task_pool_name))
         self.task_pools[task_pool_name] = TaskPool(task_pool_name, self)
 
@@ -1848,13 +1804,16 @@ class ServicesProxy:
         task_pool.terminate_tasks()
         del self.task_pools[task_pool_name]
 
-    def create_sub_workflow(self, sub_name, config_file, override={}, input_dir=None):
+    def create_sub_workflow(self, sub_name, config_file, override=None, input_dir=None):
         """Create sub-workflow
 
         """
 
-        if sub_name in list(self.sub_flows.keys()):
-            self.exception("Duplicate sub flow name")
+        if override is None:
+            override = {}
+
+        if sub_name in self.sub_flows:
+            self.error("Duplicate sub flow name")
             raise Exception("Duplicate sub flow name")
 
         self.subflow_count += 1
@@ -1862,12 +1821,12 @@ class ServicesProxy:
             sub_conf_new = ConfigObj(infile=config_file, interpolation='template', file_error=True)
             sub_conf_old = ConfigObj(infile=config_file, interpolation='template', file_error=True)
         except Exception:
-            self.exception("Error accessing sub-workflow config file %s" % config_file)
+            self.exception("Error accessing sub-workflow config file %s", config_file)
             raise
         # Update undefined sub workflow configuration entries using top level configuration
         # only applicable to non-component entries (ones with non-dictionary values)
         for (k, v) in self.sim_conf.items():
-            if k not in list(sub_conf_new.keys()) and type(v).__name__ != 'dict':
+            if k not in sub_conf_new and type(v).__name__ != 'dict':
                 sub_conf_new[k] = v
 
         sub_conf_new['SIM_NAME'] = self.sim_name + "::" + sub_name
@@ -1926,11 +1885,9 @@ class TaskPool:
     """
     Class to contain and manage a pool of tasks.
     """
-    dask = None
-    distributed = None
     try:
-        dask: dask = __import__("dask")
-        distributed: dask.distributed = __import__("dask.distributed")
+        dask = __import__("dask")
+        distributed = __import__("dask.distributed")
     except ImportError:
         dask = None
         distributed = None
@@ -1956,6 +1913,7 @@ class TaskPool:
         self.futures = None
         self.dask_file_name = None
         self.dask_client = None
+        self.worker_event_logfile = None
 
     def _wait_any_task(self, block=True):
         """
@@ -2019,7 +1977,7 @@ class TaskPool:
             except KeyError:
                 binary_fullpath = ipsutil.which(binary)
             if not binary_fullpath:
-                self.services.error("Program %s is not in path or is not executable" % binary)
+                self.services.error("Program %s is not in path or is not executable", binary)
                 raise Exception("Program %s is not in path or is not executable" % binary)
             else:
                 self.services.binary_fullpath_cache[binary] = binary_fullpath
@@ -2066,7 +2024,6 @@ class TaskPool:
                                                          "--nprocs", 1,
                                                          "--nthreads", nthreads,
                                                          "--no-dashboard",
-                                                         "--preload", self.services.dask_preload,
                                                          task_ppn=1)
         else:
             self.dask_workers_tid = services.launch_task(dask_nodes, os.getcwd(),
@@ -2076,7 +2033,6 @@ class TaskPool:
                                                          "--nprocs", 1,
                                                          "--nthreads", nthreads,
                                                          "--no-dashboard",
-                                                         "--preload", self.services.dask_preload,
                                                          task_ppn=1)
 
         self.dask_client = self.dask.distributed.Client(scheduler_file=self.dask_file_name)
@@ -2130,7 +2086,7 @@ class TaskPool:
             if TaskPool.dask and self.serial_pool:
                 self.dask_pool = True
                 if use_shifter and not self.shifter:
-                    self.services.exception("Requested to run dask within shifter but shifter not available")
+                    self.services.error("Requested to run dask within shifter but shifter not available")
                     raise Exception("shifter not found")
                 else:
                     return self.submit_dask_tasks(block, dask_nodes, dask_ppn, use_shifter)
@@ -2237,7 +2193,7 @@ class TaskPool:
                 _ = [f.cancel() for f in self.futures]
                 self.futures = []
             else:
-                for task_id in list(self.active_tasks.keys()):
+                for task_id in self.active_tasks:
                     self.services.kill_task(task_id)
         self.queued_tasks = {}
         self.blocked_tasks = {}
