@@ -26,6 +26,7 @@ class TaskManager:
         self.data_mgr = None
         self.resource_mgr = None
         self.config_mgr = None
+        self.host = None
         self.comp_registry = configurationManager.ComponentRegistry()
         self.service_methods = ['init_call',
                                 'launch_task',
@@ -71,7 +72,6 @@ class TaskManager:
         self.resource_mgr = resource_mgr
         self.config_mgr = config_mgr
         self.host = self.config_mgr.get_platform_parameter('HOST')
-        self.node_alloc_mode = self.config_mgr.get_platform_parameter('NODE_ALLOCATION_MODE')
         try:
             self.task_launch_cmd = self.config_mgr.get_platform_parameter('MPIRUN')
         except Exception:
@@ -102,9 +102,9 @@ class TaskManager:
         Prints the task table pretty-like.
         """
         ctt = self.curr_task_table
-        for c, i in list(ctt.items()):
+        for c, i in ctt.items():
             print(c)
-            for k, v in list(i.items()):
+            for k, v in i.items():
                 print("   ", k, "=", v)
             print("------")
         print("=====================")
@@ -222,25 +222,13 @@ class TaskManager:
         block = init_task_msg.args[4]  # Block waiting for available resources
         wnodes = init_task_msg.args[5]
         wsocks = init_task_msg.args[6]
+        tcpp = init_task_msg.args[7]
 
         # SIMYAN: increased arguments
-        cmd_args = init_task_msg.args[7:]
-        # handle for task related things
-        task_id = self.get_task_id()
+        cmd_args = init_task_msg.args[8:]
 
         try:
-            retval = self.resource_mgr.get_allocation(caller_id,
-                                                      nproc,
-                                                      task_id,
-                                                      wnodes,
-                                                      wsocks,
-                                                      task_ppn=tppn)
-            self.fwk.debug('RM: get_allocation() returned %s', str(retval))
-            partial_node = retval[0]
-            if partial_node:
-                (nodelist, corelist, ppn, max_ppn, accurateNodes) = retval[1:]
-            else:
-                (nodelist, ppn, max_ppn, accurateNodes) = retval[1:]
+            return self._init_task(caller_id, nproc, binary, working_dir, tppn, tcpp, wnodes, wsocks, cmd_args)
         except InsufficientResourcesException:
             if block:
                 raise BlockedMessageException(init_task_msg, '***%s waiting for %d resources' %
@@ -258,16 +246,24 @@ class TaskManager:
         except Exception:
             raise
 
-        # SIMYAN: moved up a few lines and ret_data, node_file added
-        self.curr_task_table[task_id] = {'component': caller_id,
-                                         'status': 'init_task',
-                                         'binary': binary,
-                                         'nproc': nproc,
-                                         'args': cmd_args,
-                                         'launch_cmd': None,
-                                         'env_update': None,
-                                         'ret_data': None,
-                                         'node_file': None}
+    def _init_task(self, caller_id, nproc, binary, working_dir, tppn, tcpp, wnodes, wsocks, cmd_args):
+        # handle for task related things
+        task_id = self.get_task_id()
+
+        retval = self.resource_mgr.get_allocation(caller_id,
+                                                  nproc,
+                                                  task_id,
+                                                  wnodes,
+                                                  wsocks,
+                                                  task_ppn=tppn,
+                                                  task_cpp=tcpp)
+        self.fwk.debug('RM: get_allocation() returned %s', str(retval))
+        partial_node = retval[0]
+        if partial_node:
+            (nodelist, corelist, ppn, max_ppn, accurateNodes) = retval[1:]
+        else:
+            (nodelist, ppn, max_ppn, cpp, accurateNodes) = retval[1:]
+
         if partial_node:
             nodes = ','.join(nodelist)
             (cmd, env_update) = self.build_launch_cmd(nproc, binary, cmd_args,
@@ -285,15 +281,22 @@ class TaskManager:
                                                       working_dir, ppn,
                                                       max_ppn, nodes,
                                                       accurateNodes,
-                                                      False, task_id)
-        task_data = self.curr_task_table[task_id]
-        task_data['launch_cmd'] = cmd
-        task_data['env_update'] = env_update
+                                                      False, task_id,
+                                                      cpp)
+
+        self.curr_task_table[task_id] = {'component': caller_id,
+                                         'status': 'init_task',
+                                         'binary': binary,
+                                         'nproc': nproc,
+                                         'args': cmd_args,
+                                         'launch_cmd': cmd,
+                                         'env_update': env_update}
+
         return (task_id, cmd, env_update)
 
     def build_launch_cmd(self, nproc, binary, cmd_args, working_dir, ppn,
                          max_ppn, nodes, accurateNodes, partial_nodes,
-                         task_id, core_list=''):
+                         task_id, cpp=0, core_list=''):
         """
         Construct task launch command to be executed by the component.
 
@@ -361,14 +364,14 @@ class TaskManager:
                     num_cores = self.resource_mgr.cores_per_socket
                     for (n, cl) in core_list:
                         core_dict.update({n: cl})
-                        if len(cl) in list(ppn_groups.keys()):
+                        if len(cl) in ppn_groups:
                             ppn_groups[len(cl)].append(n)
                         else:
                             ppn_groups.update({len(cl): [n]})
                     cmdlets = []
                     envlets = []
                     bin_n_args = ' '.join([binary, *cmd_args])
-                    for p, ns in list(ppn_groups.items()):
+                    for p, ns in ppn_groups.items():
                         cmdlets.append(' '.join([','.join(ns), str(p),
                                                  bin_n_args]))
                         el_node = []
@@ -504,11 +507,24 @@ class TaskManager:
             nproc_flag = '-n'
             nnodes_flag = '-N'
             num_nodes = len(nodes.split(','))
-            cmd = ' '.join([self.task_launch_cmd, nnodes_flag,
-                            str(num_nodes), nproc_flag, str(nproc)])
+            if partial_nodes:
+                cmd = ' '.join([self.task_launch_cmd,
+                                nnodes_flag, str(num_nodes),
+                                nproc_flag, str(nproc)])
+            else:
+                cpuptask_flag = '-c'
+                cpubind_flag = '--threads-per-core=1 --cpu-bind=cores'
+                cmd = ' '.join([self.task_launch_cmd,
+                                nnodes_flag, str(num_nodes),
+                                nproc_flag, str(nproc),
+                                cpuptask_flag, str(cpp),
+                                cpubind_flag])
+                env_update = {'OMP_PLACES': 'threads',
+                              'OMP_PROC_BIND': 'spread',
+                              'OMP_NUM_THREADS': str(cpp)}
         else:
-            self.fwk.exception("invalid task launch command.")
-            raise "invalid task launch command."
+            self.fwk.error("invalid task launch command.")
+            raise RuntimeError("invalid task launch command.")
 
         cmd_args = ' '.join(cmd_args)
         cmd = ' '.join([cmd, binary, cmd_args])
@@ -530,37 +546,25 @@ class TaskManager:
         caller_id = init_task_msg.sender_id
         task_dict = init_task_msg.args[0]
         ret_dict = {}
-        for task_name in list(task_dict.keys()):
+        for task_name in task_dict:
             # handle for task related things
-            task_id = self.get_task_id()
-            (nproc, working_dir, binary, cmd_args, tppn, wnodes, wsocks) = task_dict[task_name]
+            (nproc, working_dir, binary, cmd_args, tppn, wnodes, wsocks, tcpp) = task_dict[task_name]
 
             try:
-                retval = self.resource_mgr.get_allocation(caller_id,
-                                                          nproc,
-                                                          task_id,
-                                                          wnodes, wsocks,
-                                                          task_ppn=tppn)
-                self.fwk.debug('RM: get_allocation() returned %s', str(retval))
-                partial_node = retval[0]
-                if partial_node:
-                    (nodelist, corelist, ppn, max_ppn, accurateNodes) = retval[1:]
-                else:
-                    (nodelist, ppn, max_ppn, accurateNodes) = retval[1:]
-
+                ret_dict[task_name] = self._init_task(caller_id, nproc, binary, working_dir, tppn, tcpp, wnodes, wsocks, cmd_args)
             except InsufficientResourcesException:
                 continue
             except BadResourceRequestException as e:
                 self.fwk.error("There has been a fatal error, %s requested %d too many processors in task %d",
                                caller_id, e.deficit, e.task_id)
-                for (task_id, cmd) in list(ret_dict.values()):
+                for task_id, _, _ in ret_dict.values():
                     self.resource_mgr.release_allocation(task_id, -1)
                     del self.curr_task_table[task_id]
                 raise
             except ResourceRequestMismatchException as e:
                 self.fwk.error("There has been a fatal error, %s requested too few processors per node to launch task %d (request: procs = %d, ppn = %d)",
                                caller_id, e.task_id, e.nproc, e.ppn)
-                for (task_id, cmd) in list(ret_dict.values()):
+                for task_id, _, _ in ret_dict.values():
                     self.resource_mgr.release_allocation(task_id, -1)
                     del self.curr_task_table[task_id]
                 raise
@@ -568,37 +572,6 @@ class TaskManager:
                 self.fwk.exception('TM:init_task_pool(): Allocation exception')
                 raise
 
-            if partial_node:
-                nodes = ','.join(nodelist)
-                (cmd, env_update) = self.build_launch_cmd(nproc, binary,
-                                                          cmd_args,
-                                                          working_dir, ppn,
-                                                          max_ppn, nodes,
-                                                          accurateNodes,
-                                                          partial_node,
-                                                          task_id,
-                                                          core_list=corelist)
-            else:
-                if accurateNodes:
-                    nodes = ','.join(nodelist)
-                else:
-                    nodes = ''
-                (cmd, env_update) = self.build_launch_cmd(nproc, binary,
-                                                          cmd_args,
-                                                          working_dir,
-                                                          ppn, max_ppn, nodes,
-                                                          accurateNodes, False,
-                                                          task_id)
-
-            self.curr_task_table[task_id] = {'component': caller_id,
-                                             'status': 'init_task',
-                                             'binary': binary,
-                                             'nproc': nproc,
-                                             'args': cmd_args,
-                                             'launch_cmd': cmd,
-                                             'env_update': env_update,
-                                             'ret_data': None}
-            ret_dict[task_name] = (task_id, cmd, env_update)
         return ret_dict
 
     def finish_task(self, finish_task_msg):
